from django.db import models
from account.models import Account
from categories.models import Category
from uom.models import UnitOfMeasure
from suppliers.models import Supplier
from gauge.models import Gauge
import random
from django.core.exceptions import ValidationError

def generate_product_code():
    # Generate a random number with a fixed length (e.g., 6 digits)
    random_number = str(random.randint(0, 999999)).zfill(6)

    # Create the product code by concatenating "PID" and the random number
    product_code = f'PID{random_number}'

    return product_code

class Product(models.Model):
    product_id = models.AutoField(primary_key=True)
    pid = models.CharField(max_length=200, default=generate_product_code())
    added_by = models.ForeignKey(Account, on_delete=models.SET_NULL, null=True, blank=True)
    
    # Product Information 
    product_name = models.CharField(max_length=255, null=True, blank=True)
    product_description = models.TextField(null=True, blank=True)
    category = models.ForeignKey(Category, on_delete=models.CASCADE, null=True, blank=True)
    uom = models.ForeignKey(UnitOfMeasure, on_delete=models.CASCADE, null=True, blank=True)
    gauge = models.ForeignKey(Gauge, on_delete=models.CASCADE, null=True, blank=True )
    FINISH_CHOICES = [
        ('none', 'None'),
        ('matte', 'Matte'),
        ('glossy', 'Glossy'),
    ]
    finish = models.CharField(max_length=50, choices=FINISH_CHOICES,blank=True, null=True)
    supplier_id = models.ForeignKey(Supplier, on_delete=models.SET_NULL, null=True, blank=True)
    manufacturer = models.CharField(max_length=100, null=True, blank=True)
    creation_date = models.DateField(null=True, blank=True)
    last_updated_date = models.DateField(null=True, blank=True)
    images = models.ImageField(upload_to='product_images/', null=True, blank=True)
    active_status = models.BooleanField(null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True, null=True, blank=True)
    updated_at = models.DateTimeField(auto_now=True, null=True, blank=True)
    
    # Pricing Information
    price = models.DecimalField(max_digits=10, decimal_places=2, null=True, blank=True)
    min_price = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    discounted_price = models.DecimalField(max_digits=10, decimal_places=2, null=True, blank=True)

    # Profiling Information
    is_profiling = models.BooleanField(default=False)   
    profiling_price = models.DecimalField(max_digits=10, decimal_places=2, default=0)  

    # Stock Information
    sku = models.CharField(max_length=20, null=True, blank=True)
    barcode = models.CharField(max_length=20, null=True, blank=True)
    quantity_in_stock = models.IntegerField(default=0, null=True, blank=True)
    min_stock_level = models.IntegerField(default=0, null=True, blank=True)
    max_stock_level = models.IntegerField(default=0, null=True, blank=True)
    reorder_point = models.IntegerField(default=0, null=True, blank=True)

    # Packaging Information
    weight = models.DecimalField(max_digits=10, decimal_places=2, null=True, blank=True)
    dimensions = models.CharField(max_length=50, null=True, blank=True)
    
    # Product Type
    PRODUCT_TYPE_CHOICES = [
        ('Production Item', 'Production Item'),
        ('Trading Item', 'Trading Item'),
    ]
    productType = models.CharField(
        max_length=50,
        null=True,
        blank=True,
        choices=PRODUCT_TYPE_CHOICES,
        default="ProductionItem"
    )

    def save(self, *args, **kwargs):
        # Check if the product is being updated (already exists in the database)
        if self.pk:
            # Ensure quantity in stock is greater than zero for updates
            if self.quantity_in_stock < 0:
                raise ValidationError('Quantity in stock must be greater than zero on update.')
        
        # Ensure quantity in stock is never below zero when saving
        if self.quantity_in_stock < 0:
            raise ValidationError('Quantity in stock cannot be less than 0.')

        super(Product, self).save(*args, **kwargs)

    def __str__(self):
        return self.product_name or "Unnamed Product"

